#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2021-09-08 10:13:01
# @Author  : wangmian05
# @Link    : wangmian05@countrygraden.com.cn
# @Version : $Id$
# 统一返回结果
status_success = 0
status_error = 1


class Result:
    # 任务id
    taskId = None
    # 状态
    status = None
    # 消息
    msg = None

    def __init__(self, taskId, status=status_success, message=""):
        self.taskId = taskId
        self.status = status
        self.message = message

    def success(self, taskId):
        self.taskId = taskId
        self.status = status_success
        return self.__dict__

    def error(self, taskId, msg):
        self.taskId = taskId
        self.status = status_error
        self.msg = msg
        return self.__dict__



import json

from aisrfwk.util import log as log, genid, redis_util
from aisrfwk.util import redis_util as redis


# 新任务处理加入队列
def pushQueue(content):
    log.info("push new task:{}", str(content)[0:100])
    task_id = buildTaskId(content)
    content["taskId"] = task_id
    redis_util.putTask(task_id, json.dumps(content))
    return {"taskId": task_id}


def buildTaskId(content):
    worker = genid.IdWorker(1, 1, 0)
    pro_id = content.get("projectId")
    if pro_id:
        task_id = str(pro_id) + str(worker.get_id())
    else:
        task_id = str(worker.get_id())
    content["taskId"] = task_id
    return task_id



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2021-09-14 16:30:50
# @Author  : wangmian05 (wangmian05@countrygraden.com.cn)
# @Link    : https://git.bgy.com.cn/bu00439/aisr-carbarn-ai.git
# @Version : $Id$
import json

from aisrfwk.rocketmq.rocketmq_core import sendMsg
from aisrfwk.util.config import config
from aisrfwk.util import log as log

system_name = config.get("system.name")

aisr_result_plan = config.get("mq.topic")

result_status = {"success": 0, "error": 1}


def sendErrorToServer(task_id, msg):
    """
    推送任务执行失败到mq
    """
    result_topic = aisr_result_plan
    msg = {
        "taskId": task_id,
        "status": result_status.get("error"),
        "msg": msg
    }
    sendMsg(result_topic, json.dumps(msg))
    log.info("[rocketmq]{}方案出错,错误消息发送成功...", system_name)


def sendResultToServer(content={}):
    """
    推送任务执行成功,附带结果数据到mq
    """
    result_topic = aisr_result_plan
    # 推入mq的数据.java端只取非坐标类的数据.坐标类的数据通过redis获取
    content["status"] = result_status.get("success")
    content["lastPlan"] = False
    sendMsg(result_topic, json.dumps(content))
    log.info("[rocketmq]{}方案数据消息发送成功...", system_name)


def sendTaskOverToServer(content={}, status="success"):
    """
    发送方案生成结束的标志消息去的到后端
    """
    result_topic = aisr_result_plan
    content["status"] = result_status.get("error") if status != "success" else result_status.get("success")
    content["lastPlan"] = True
    sendMsg(result_topic, json.dumps(content))

    log.info("[rocketmq]{}方案生成结束消息发送成功,内容:{}...", system_name, json.dumps(content))



import time

from aisrfwk.domain.base_aisr_task import BaseAisrTask
from aisrfwk.util import log as log


def Log(fn):
    """异步执行方法"""

    def wrapper(*args, **kwargs):
        log.info("{}方法开始执行", fn.__name__)
        method_start = time.time()
        res = fn(*args, **kwargs)
        method_end = time.time()
        methodTime = round(method_end - method_start, 2)
        log.info("{}方法执行完成,总耗时:[{}]秒...", fn.__name__, methodTime)
        return res

    return wrapper


def BussinesTask(fn):
    def wrapper(*args, **kwargs):
        log.info("{}标注", fn.__name__)
        b = BaseAisrTask()
        b.addSub(fn.__name__)

    return wrapper



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2021-09-14 16:30:50
# @Author  : wangmian05 (wangmian05@countrygraden.com.cn)
# @Link    : https://git.bgy.com.cn/bu00439/aisr-carbarn-ai.git
# @Version : $Id$
import json
import sys

from aisrfwk.util.config import config
from aisrfwk.util import log as log
from rocketmq.client import Producer

result_status = {"success": 0, "error": 1}

mq_message_size = 0


def buildProducer():
    producer = None
    try:
        # params_dict = {"id": task_id, "json": 0}
        # 建立MQ连接，配置标签
        producer = Producer(config.get("mq.producer", "aisr-fwk"), compress_level=9,
                            max_message_size=1024 * 1024 * 4)
        producer.set_namesrv_addr(config.get("mq.addr", "0.0.0.0:9876"))
        # 修改最大消息限制为2M
        producer.set_max_message_size(1024 * 1024 * 4)
        producer.start()
        log.info("producer start......")

    except:
        log.error("rocketmq producer start fail...{}", sys.exc_info())
        producer.shutdown()
    return producer


def sendMsg(topic, val):
    global mq_message_size
    if not bool(topic):
        log.error("消息发送失败,topic不能为空")
        return
    if not bool(val):
        log.error("消息发送失败,消息内容不能为空")
        return
    log.info("开始往topic:[{}],推送mq消息", topic)
    # log.info("开始往topic:[{}],推送mq消息:{}", topic, val)

    count = 0
    success = False
    while not success and count < 3:
        count += 1
        producer = None
        try:
            from rocketmq.client import Message, Producer
            msg = Message(topic)
            msg.set_tags('*')
            msg.set_body(val)
            # 发送信息,暂时使用同步发送
            producer = buildProducer()
            if producer:
                _p = json.dumps(producer.__dict__)
            else:
                log.error("生成者对象为空!!!")
            log.info("mq-size:{},开始发送消息:{}......", mq_message_size, val)
            # 发送一次消息本地记录一次,当数量大于一定值后归零,这个数量会影响到消息投递的队列
            # ret = producer.send_orderly(msg, mq_message_size, retry_times=30)
            ret = producer.send_sync(msg)
            log.info("消息发送完成.....")
            mq_message_size = 0 if mq_message_size > 10000000 else mq_message_size + 1
            log.info("消息是否发送成功:{},消息id:{}", mq_message_size, ret.status, ret.msg_id)
            success = True
            producer.shutdown()
        except:
            log.error("开始第[{}]此重试...消息发送失败:{}", count, sys.exc_info())
        finally:
            if producer is not None:
                producer.shutdown()

    log.info("推送mq消息完成:{}", success)
    if not success:
        log.error("重试三次都失败...不在重新发送")
        raise Exception("mq消息发送失败,重试三次都失败")



import sys
import json

from aisrfwk.util import log
from aisrfwk.domain.base_result import Result
from flask import Flask, jsonify, request
from flask_cors import cross_origin
from aisrfwk.util.config import config, service_port
import aisrfwk.util.redis_util as redis

from aisrfwk.util import api

sys.path.append("..")

app = Flask(__name__)


@app.route('/')
@cross_origin()
def ping():
    return jsonify('pong')


@app.route('/park', methods=['POST'])
@cross_origin()
def park():
    """
    车位排布主方法
    """
    try:
        param = request.get_data()
        json_param = json.loads(param)
        validateParam(json_param)
        res = api.pushQueue({"test": "name"})
        return Result().success(res)
    except Exception as e:
        log.info(type(e))
        log.error("车位排布任务启动失败:" + str(e))
        return Result().error(str(e))


def validateParam(param):
    """根据图纸生成方案参数校验"""
    assert param.get("projectId") is not None, '项目ID参数[projectId]必须传入'
    # assert param.get("params") is not None, '指标参数[params]必须传入'
    # assert param.get("outer") is not None, '地库退线参数[outer]必须传入'
    # assert param.get("building") is not None, '楼栋外轮廓参数[outer]必须传入'
    # assert param.get("column") is not None, '剪力墙参数[column]必须传入'
    # assert param.get("coreBarrel") is not None, '核心筒参数[coreBarrel]必须传入'


def scheduleTask():
    try:
        log.debug("数据状态检测定时任务开始运行...")
        redis.restartTodoRunTask()
        # t = Timer(60*2, scheduleTask)
        # t.start()
    except Exception as e:
        log.error("当前定时任务运行出错:{}", str(e))


def start():
    """启动web服务"""
    server_host = config.get("server.host", "0.0.0.0")
    server_port = service_port if service_port != 0 else config.get("server.port", 5000)
    log.info("当前配置.端口号:{},ip:{}", server_host, server_port)
    # scheduleTask()
    app.run(host=server_host, port=server_port)


if __name__ == '__main__':
    # 本地运行Flask
    # po = Pool(10)  # 定义一个进程池，最大进程数10
    # 启动api监听
    start()
    # 启动队列监听
    # po.close( )
    # po.join()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2021-09-14 11:33:17
# @Author  : wangmian05 (wangmian05@countrygraden.com.cn)
# @Link    : https://git.bgy.com.cn/bu00439/aisr-carbarn-ai.git
# @Version : $Id$

import os


import time
import logging

# 64位ID的划分
WORKER_ID_BITS = 5
DATACENTER_ID_BITS = 5
SEQUENCE_BITS = 12

# 最大取值计算
MAX_WORKER_ID = -1 ^ (-1 << WORKER_ID_BITS)  # 2**5-1 0b11111
MAX_DATACENTER_ID = -1 ^ (-1 << DATACENTER_ID_BITS)

# 移位偏移计算
WOKER_ID_SHIFT = SEQUENCE_BITS
DATACENTER_ID_SHIFT = SEQUENCE_BITS + WORKER_ID_BITS
TIMESTAMP_LEFT_SHIFT = SEQUENCE_BITS + WORKER_ID_BITS + DATACENTER_ID_BITS

# 序号循环掩码
SEQUENCE_MASK = -1 ^ (-1 << SEQUENCE_BITS)

# Twitter元年时间戳
TWEPOCH = 1288834974657


logger = logging.getLogger('flask.app')


class IdWorker(object):
    """
    用于生成IDs
    """

    def __init__(self, datacenter_id, worker_id, sequence=0):
        """
        初始化
        :param datacenter_id: 数据中心（机器区域）ID
        :param worker_id: 机器ID
        :param sequence: 其实序号
        """
        # sanity check
        if worker_id > MAX_WORKER_ID or worker_id < 0:
            raise ValueError('worker_id值越界')

        if datacenter_id > MAX_DATACENTER_ID or datacenter_id < 0:
            raise ValueError('datacenter_id值越界')

        self.worker_id = worker_id
        self.datacenter_id = datacenter_id
        self.sequence = sequence

        self.last_timestamp = -1  # 上次计算的时间戳

    def _gen_timestamp(self):
        """
        生成整数时间戳
        :return:int timestamp
        """
        return int(time.time() * 1000)

    def get_id(self):
        """
        获取新ID
        :return:
        """
        timestamp = self._gen_timestamp()

        # 时钟回拨
        if timestamp < self.last_timestamp:
            logging.error('clock is moving backwards. Rejecting requests until{}'.format(self.last_timestamp))
            raise Exception

        if timestamp == self.last_timestamp:
            self.sequence = (self.sequence + 1) & SEQUENCE_MASK
            if self.sequence == 0:
                timestamp = self._til_next_millis(self.last_timestamp)
        else:
            self.sequence = 0

        self.last_timestamp = timestamp

        new_id = ((timestamp - TWEPOCH) << TIMESTAMP_LEFT_SHIFT) | (self.datacenter_id << DATACENTER_ID_SHIFT) | \
                 (self.worker_id << WOKER_ID_SHIFT) | self.sequence
        return new_id

    def _til_next_millis(self, last_timestamp):
        """
        等到下一毫秒
        """
        timestamp = self._gen_timestamp()
        while timestamp <= last_timestamp:
            timestamp = self._gen_timestamp()
        return timestamp


def test():
    for i in range(10):
        id = worker.get_id()
        print(id)


if __name__ == '__main__':
    from threading import Thread
    worker = IdWorker(1, 1, 0)
    l = list()
    for i in range(2):
        t = Thread(target=test)
        t.start()
        l.append(t)
    for t in l:
        t.join()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2021-09-09 14:49:16
# @Author  : wangmian05 (wangmian05@countrygraden.com.cn)
# @Link    : https://git.bgy.com.cn/bu00439/aisr-carbarn-ai.git
# @Version : $Id$

import time
from multiprocessing import Process

from aisrfwk.util import log as log

max_thread = 32


class ProcessPool:
    process_list = []
    max_process = 0

    def __init__(self, _max_process=max_thread):
        self.max_process = _max_process

    def asyncProcess(self, f, *args):
        """异步执行方法"""

        process = Process(target=f, args=args)
        process.start()
        self.process_list.append(process)
        if len(self.process_list) >= max_thread:
            # 查找运行态的线程,如果线程大于最大线程数.则停止消费队列任务,交给其他机器处理.
            run_list = [rl for rl in self.process_list if not rl._closed]
            _run_list = [rl for rl in run_list if rl.is_alive()]
            # 如果所有线程都在处理.则需要等待处理完成后继续处理
            if len(_run_list) >= max_thread:
                self.__wait_process(_run_list)
                log.info("当前线程池已经恢复空闲,开始处理任务")
            # _not_run_list = [rl for rl in run_list if rl._closed]
            # # 结束掉已经stop的进程
            # for nr in _not_run_list:
            #     nr.close()
            self.process_list = [rl for rl in run_list if not rl._closed or rl.is_alive()]

    # 进程等待主方法
    @staticmethod
    def __wait_process(_run_list):
        _run_list = [rl for rl in _run_list if not rl._closed]
        _not_run_list = [rl for rl in _run_list if not rl.is_alive()]
        while len(_not_run_list) == 0:
            log.info("当前线程池已满.等待所有线程执行完毕后再次启动")
            time.sleep(2)
            _not_run_list = [rl for rl in _run_list if not rl.is_alive()]


# 进程池对象
pool = ProcessPool()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2021-09-06 15:52:46
# @Author  : wangmian05
# @Link    : wangmian05@countrygraden.com.cn
# @Version : $Id$
import json
import time
import redis
from aisrfwk.util.config import config
from aisrfwk.util import log as log
from aisrfwk.util.mail_util import sendMail

env_prefix = config.get("env") + ":" if "env" in config and len(config.get("env")) > 0 else ""
global_key = config.get("redis.global_key","aisr-fwk")

task_status = {"todo": 1, "running": 2, "success": "3", "error": "4"}

conn_pool = redis.ConnectionPool(host=config.get("redis.host"), port=config.get("redis.port"),
                                 db=config.get("redis.db"), password=config.get("redis.passwd"))
re_pool = redis.Redis(connection_pool=conn_pool, health_check_interval=30, socket_keepalive=30)


def setStr(key, val):
    return re_pool.set(global_key +":"+ key, val)


def getStr(key):
    return str(re_pool.get(global_key +":"+ key))


def setHash(key, hkey, hval):
    return re_pool.hset(key, hkey, hval)


def getHash(key, hkey):
    return str(re_pool.hget(key, hkey))


def lPush(key, val):
    return re_pool.lpush(key, str(val))


def rPop(key):
    sss = re_pool.rpop(key)
    return str(sss.decode("utf-8")) if sss is not None else ''


# 先进先出.阻塞队列出的内容为元组类型,0=队列key,1=队列值
def bRpop(key):
    tup = re_pool.brpop(key, timeout=30)
    if tup is None:
        return
    val = tup[1]
    return str(val.decode("utf-8")) if val is not None else ''



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2021-09-08 10:13:01
# @Author  : wangmian05
# @Link    : wangmian05@countrygraden.com.cn
# @Version : $Id$
# 统一返回结果
res_success_msg = "算法服务请求成功"
res_error_msg = "算法服务请求失败"

result_status = {"success": 0, "error": 1}


class Result:
    # 状态
    status = None
    # 状态
    message = None
    # 状态
    result = None

    def __init__(self, status=result_status.get("success"), message="", result={}):
        self.status = status
        self.message = message
        self.result = result

    def success(self, result):
        self.status = result_status.get("success")
        self.message = res_success_msg
        self.result = result
        return self.__dict__

    def error(self, msg):
        self.status = result_status.get("error")
        self.message = "{}:{}".format(res_error_msg, msg)
        self.result = {}
        return self.__dict__



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2021-09-09 14:49:16
# @Author  : wangmian05 (wangmian05@countrygraden.com.cn)
# @Link    : https://git.bgy.com.cn/bu00439/aisr-carbarn-ai.git
# @Version : $Id$

import os

from threading import Thread
from time import sleep


def asyncMethod(f):
    """异步执行方法"""

    def wrapper(*args, **kwargs):
        thr = Thread(target=f, args=args, kwargs=kwargs)
        thr.start()

    return wrapper



import logging
import os
import sys
import time

# 创建日志
logging.getLogger("shapely").setLevel(logging.CRITICAL)
logger = logging.getLogger()

# 设置日志
# 等级总开关的默认级别为WARNING，此处改为DEBUG
logger.setLevel(logging.DEBUG)

env = sys.argv[1] if len(sys.argv) > 1 else None
env_prefix = env if env and len(env) > 0 else "local"
if env_prefix != "local":
    # 文件handler
    log_time = time.strftime('%Y%m%d', time.localtime(time.time()))
    log_path = os.path.join(os.getcwd(), 'logs/')
    if not os.path.exists(log_path):
        os.mkdir(log_path)
    log_file_name = log_path + log_time + '.log'
    file_handler = logging.FileHandler(log_file_name)
    # 设置文件handler的等级
    file_handler.setLevel(logging.ERROR)
    # 第三步，定义handler的输出格式
    file_formatter = logging.Formatter(
        "%(asctime)s - %(filename)s[line:%(lineno)d] - pid:%(process)d - tid:%(thread)d - %(levelname)s: %(message)s")
    file_handler.setFormatter(file_formatter)
    logger.addHandler(file_handler)

# 控制台handler
console_handler = logging.StreamHandler()
console_handler.setLevel(logging.INFO)
console_formatter = logging.Formatter("%(asctime)s - %(levelname)s: %(message)s")
console_handler.setFormatter(console_formatter)
logger.addHandler(console_handler)


def debug(msg, *args):
    pid = os.getpid()
    if msg is None:
        return
    if len(args) > 0:
        logger.debug("process [{}] :".format(pid) + str(msg).format(*args))
    else:
        logger.debug("process [{}] :".format(pid) + str(msg))


def info(msg, *args):
    if msg is None:
        return
    pid = os.getpid()
    if len(args) > 0:
        logger.info("process [{}] :".format(pid) + str(msg).format(*args))
    else:
        logger.info("process [{}] :".format(pid) + str(msg))


def warning(msg, *args):
    pid = os.getpid()
    if msg is None:
        return
    if len(args) > 0:
        logger.warning("process [{}] :".format(pid) + str(msg).format(*args))
    else:
        logger.warning("process [{}] :".format(pid) + str(msg))


def error(msg, *args):
    if msg is None:
        return
    pid = os.getpid()
    if len(args) > 0:
        pid = os.getpid()
        logger.error("process [{}] :".format(pid) + str(msg).format(*args), exc_info=True)
    else:
        logger.error("process [{}] :".format(pid) + str(msg))


# File "/data/deploy/aisr-carbarn-ai/util/log.py", line 53, in info
#   logger.info(("process [{}] :" + msg).format(pid))

def critical(msg, *args):
    logger.critical(msg)


if __name__ == "__main__":
    debug('this is a logger debug message{}')
    info('this is a logger info message,{}', 233)
    warning('this is a logger warning message:', 12)
    error('this is a logger error message')
    critical('this is a logger critical message')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2021-09-07 17:07:08
# @Author  : wangmian05
# @Link    : wangmian05@countrygraden.com.cn
# @Version : $Id$
import json
import multiprocessing
import sys
import threading
import time

from aisrfwk.domain.base_aisr_task import BaseAisrTask
from aisrfwk.redis import redis_core
from aisrfwk.util import log, redis_util
from aisrfwk.util.config import config
from aisrfwk.util.exception_util import formatException, BaseAisrException
from aisrfwk.util.proces_util import pool

exec_count = 0
count_lock = multiprocessing.Lock()

system_name = config.get("system.name")


def __runningTask(content, model):
    """
    任务执行器
    """
    data = None
    try:
        try:
            data = json.loads(content)
            task_id = data.get("taskId")
            log.info("consumer start task:{}...", task_id)
            redis_util.runningTask(task_id)
            if model:
                model.run(data)
            else:
                BaseAisrTask().run(data)
            redis_util.overTaskStatus(task_id)
            log.info("consumer task over success:{}...", task_id)
        except BaseAisrException as bae:
            log.error("【严重】{}任务处理失败,业务异常{}:", system_name, sys.exc_info())
            task_id = data.get("taskId")
            _exc_text = "【严重】" + system_name + "任务处理失败,业务自定义异常:<br />" + formatException(sys.exc_info())
            redis_util.errorTaskStatus(task_id, _exc_text)
        except BaseException as e1:
            log.error("【严重】{}任务处理失败,未知异常{}:", system_name, sys.exc_info())
            task_id = data.get("taskId")
            _exc_text = "【严重】" + system_name + "任务处理失败,发生未知异常:<br />" + formatException(sys.exc_info())
            redis_util.errorTaskStatus(task_id, _exc_text)
            # rocketmq_util.sendErrorToServer(task_id, "【严重】车位排布任务处理过程中发生严重错误")
    except BaseException as e:
        log.error("【超级严重】当前进程发生错误,导致任务不执行,请检查任务状态:{}", sys.exc_info())


def __handleTask(content, model):
    """
    任务分发器
    """
    task_id = None
    data = None
    try:
        data = json.loads(content)
        task_id = data.get("taskId")
        log.info("consume queue task:{} running...", task_id)
        redis_util.startTask(task_id)
        pool.asyncProcess(__runningTask, content, model)
    except:
        log.error("{} queue error.redis队列数据读取解析失败:{}", system_name, sys.exc_info())
        if task_id is not None:
            _exc_text = "【严重】" + system_name
            "队列发生严重异常,redis队列数据读取解析失败:<br />" + formatException(sys.exc_info())
            redis_util.errorTaskStatus(task_id, _exc_text)
            redis_util.overTaskStatus(task_id)
            project_id = data.get("projectId") if data is not None else 0
            # rocketmq_util.sendErrorToServer(task_id, project_id, "【严重】车位排布任务进程池处理过程发生严重异常")


def __listener_task(model):
    """
    任务监听器
    """
    log.info("queue consumer start...", )
    while True:
        try:
            log.debug("queue consumer start...", )
            content = redis_core.bRpop(redis_util.aisr_queue_key)
            if content is None:
                continue
            global exec_count
            count_lock.acquire()
            exec_count += 1
            count_lock.release()
            log.info("listener start,todo:[{}] task", exec_count)
            __handleTask(content, model)
        except:
            log.debug("this rpop throw exception,retry rpop:{}", sys.exc_info())
            time.sleep(5)
            continue
        # 所有任务队列的时候都需要停顿500毫秒.给其他机器一点机会
        time.sleep(0.5)


# 线程启动方法
def start(__model=None):
    # 此处单独开一个线程监听队列任务.不能使用进程的方式,否则会导致mq发送消息是出现发送失败
    threading.Thread(target=__listener_task, args=[__model]).start()


if __name__ == "__main__":
    start()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2021-09-06 15:52:46
# @Author  : wangmian05
# @Link    : wangmian05@countrygraden.com.cn
# @Version : $Id$
import os
import sys
import smtplib
from email.header import Header
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from aisrfwk.util.config import config, env_prefix
from aisrfwk.util import log as log

encoding = "utf-8"
subject = config.get("mail.subject")
mail_title = config.get("mail.title")
send_user_name = config.get("mail.send.user")
to_user_name = config.get("mail.to.user")


def sendMail(message, attach_content=None, _subject=subject, _to_users=None, _project_name=None, _creator=None):
    """发送邮件"""

    try:
        receivers = config.get("mail.to.users")
        if receivers is not None:
            receivers = str.split(receivers, ",")

        if _to_users is None:
            _to_users = receivers
        if _to_users is None:
            log.warning("接收人为空,不再发送邮件...")
            return
        if message is None:
            log.warning("邮件发送内容为空")
            return

        mail_sender = config.get("mail.user")
        if mail_sender is None:
            log.info("配置获取为空,不在发送邮件")
            return
        mail_passwd = config.get("mail.passwd")
        mail_host = config.get("mail.host")
        mail_port = config.get("mail.port")
        if env_prefix == "dev":
            _subject = "[开发环境]-" + _subject
        if env_prefix == "uat":
            _subject = "[测试环境]-" + _subject
        if env_prefix == "prod":
            _subject = "[生产环境]-" + _subject
        smtpObj = smtplib.SMTP(mail_host, mail_port)
        smtpObj.starttls()
        smtpObj.login(mail_sender, mail_passwd)
        _msg = "<h3>" + mail_title + "</h3>"
        if _project_name:
            _msg = _msg + "<p>项目名称:  <b>" + _project_name + "</b></p>"
        if _creator:
            _msg = _msg + "<p>创建人:  <b>" + _creator + "</b></p>"
        if message:
            _msg = _msg + "<p>  " + message + "</p>"

        # 创建一个带附件的实例
        message = MIMEMultipart()
        message.attach(MIMEText(_msg, "html", encoding))
        if attach_content:
            # 构造附件1，传送当前目录下的 test.txt 文件
            attach_file = open('error.json', 'wb+')
            file_cont = str.encode(attach_content, encoding)
            attach_file.write(file_cont)
            attach_file.close()
            att1 = MIMEText(file_cont, 'base64', encoding)
            att1["Content-Type"] = 'application/octet-stream'
            # 这里的filename可以任意写，写什么名字，邮件中显示什么名字
            att1["Content-Disposition"] = 'attachment; filename="error.json"'
            message.attach(att1)

        message["From"] = Header(send_user_name, encoding)
        message["To"] = Header(to_user_name, encoding)
        message["Subject"] = Header(_subject, encoding)
        smtpObj.sendmail(mail_sender, _to_users, message.as_string())
        os.remove('error.json')
        log.info("邮件发送成功...")
    except:
        log.error("邮件发送失败:{}", sys.exc_info())


if __name__ == "__main__":
    sendMail("我也不知道啥错误...", "{\"building\": [[[92356, -81123], [92356, -88623]]]}")
    print("证明邮件发送是异步发送的")



import sys
import time

from aisrfwk.domain.base_aisr_task import BaseAisrTask
# from aisrfwk.util import log, redis_util, rocketmq_util
from aisrfwk.util.exception_util import formatException


class BusinessDemo(BaseAisrTask):

    def run(self, data):
        start_time = time.time()

        task_id = None
        try:
            # log.info("业务方法处理逻辑")
            print("xxxx")
            # 过程中处理结果发送到服务端
            # rocketmq_util.sendResultToServer()
        except BaseException as e:
            _msg = "车位排布任务失败,请检查底图和图层选择后再次重试!"
            _err_msg = "【严重】车位排布任务失败,内圈车位排布过程中出现异常:"
            # log.error(_err_msg + "{}", str(e))
            # exc_text = _err_msg + "<br />" + formatException(sys.exc_info())
            # redis_util.errorTaskStatus(task_id, exc_text)
            # rocketmq_util.sendErrorToServer()
        # 处理结束后发送消息到后端服务
        time.sleep(1)
        # rocketmq_util.sendTaskOverToServer()

        end_time = time.time()
        # log.info("所有车位排布生成完成,总耗时:[{}]秒", round(end_time - start_time, 2))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2021-12-21 15:52:46
# @Author  : wangmian05
# @Link    : wangmian05@countrygraden.com.cn
# @Version : $Id$
import io
import json
import traceback


# ============基础异常类================
class BaseAisrException(Exception):
    """
    基础业务异常
    """

    def __init__(self, msg):
        self.message = msg


# ============异常处理方法================

def formatException(ei):
    """
    Format and return the specified exception information as a string.

    This default implementation just uses
    traceback.print_exception()
    """
    _tab = "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"
    sio = io.StringIO()
    tb = ei[2]
    # See issues #9427, #1553375. Commented out for now.
    # if getattr(self, 'fullstack', False):
    #    traceback.print_stack(tb.tb_frame.f_back, file=sio)
    traceback.print_exception(ei[0], ei[1], tb, None, sio)
    s = sio.getvalue()
    sio.close()
    if s[-1:] == "\n":
        s = s[:-1]
    return _tab + s.replace("\n", "<br />" + _tab)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2021-09-06 17:48:59
# @Author  : wangmian05
# @Link    : wangmian05@countrygraden.com.cn
# @Version : $Id$
from aisrfwk.util import log as log
import base64
import sys
import os

sys.path.append("../..")


config = {}

yuzhi_envs = ["local", "dev", "sit", "uat", "prod"]


def start_with(cont, param):
    cont = cont.lstrip()
    if param in cont and (cont.index(param) == 0 or cont.index(param) == 1):
        return True
    return False


env = sys.argv[1] if len(sys.argv) > 1 else None
service_port = sys.argv[2] if len(sys.argv) > 2 else 0

log.debug("current env:{},service_port:{}", env, service_port)

env_prefix = env if env and len(env) > 0  else "local"

log.debug("profile:{}", env_prefix)
# current_path = sys.path[0]
#
# father_path = os.path.abspath(os.path.dirname(current_path) + os.path.sep + "..")
#
# ph = str(father_path)
pwd = sys.path[0]
# pwd = ph if ph.rindex("aisr-carbarn-ai") == 0 else ph[0:
#                                                       (ph.index("aisr-carbarn-ai") + len("aisr-carbarn-ai"))]
# if env_prefix != "local":
#     pwd = pwd + "/aisr-carbarn-ai"
case_file_path = pwd + "/server-" + env_prefix + ".conf"
if os.path.exists(case_file_path):

    with open(case_file_path, "rb") as fs:
        for f in fs:
            content = f.decode('utf-8').replace("\r", '').replace("\n", '')
            if len(content) == 0 or start_with(content, "#"):
                continue

            ct_idx = content.index("=")
            isp = [content[0:ct_idx], content[ct_idx + 1:]]
            dbc = isp[1]
            if isp[0].find("passwd") >= 0:
                try:
                    encrypt_code = base64.decodebytes(dbc[1:len(dbc) - 1].encode("utf-8")).decode()
                    isp[1] = "".join(list(base64.decodebytes(
                        "".join(list(encrypt_code)[::-1])[len(env_prefix) + 6:len(encrypt_code)].encode("utf-8")).decode())[
                        ::-1])
                except Exception as e:
                    log.warning("config read fail...")
            config[isp[0]] = isp[1]
else:
    print("config file "+env_prefix+" not found...")
#
# if __name__ == "__main__":
#     con = "xxxxxxxxxxxxxxxxxxxxxxxxxxx"
#     xxx = base64.encodebytes("".join(list(con)[::-1]).encode("utf-8")).decode()
#     prefix = "xxxxpasswd"
#     con = prefix.upper() + xxx
#     xxx = base64.encodebytes("".join(list(con)[::-1]).encode("utf-8")).decode()
#     print(xxx)
#     ddd = base64.decodebytes(xxx.encode("utf-8")).decode()
#     ccc = "".join(
#         list(base64.decodebytes("".join(list(ddd)[::-1])[len(prefix):len(ddd)].encode("utf-8")).decode())[::-1])
#     print(ccc)
#     pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2021-09-06 15:52:46
# @Author  : wangmian05
# @Link    : wangmian05@countrygraden.com.cn
# @Version : $Id$
import json
import time

from aisrfwk.redis.redis_core import re_pool, setHash, lPush
from aisrfwk.util.config import config
from aisrfwk.util import log as log
from aisrfwk.util.mail_util import sendMail

global_key = config.get("redis.global_key", "aisr-fwk")
aisr_queue_key = global_key + ":" + config.get("redis.task_queue", "task_queue")
aisr_task_status_key = global_key + ":" + config.get("redis.task_status", "task_status")
aisr_task_data_key = global_key + ":" + config.get("redis.task_data", "task_data")
aisr_send_mail_key = global_key + ":" + config.get("redis.send_mail", "send_mail")

task_status = {"todo": 1, "running": 2, "success": "3", "error": "4"}


def putTask(_task_id, content):
    data = {"status": task_status.get("todo"), "putTime": int(round(time.time() * 1000))}
    setHash(aisr_task_status_key, _task_id, json.dumps(data))
    setHash(aisr_task_data_key, _task_id, content)
    lPush(aisr_queue_key, content)


def startTask(_task_id):
    data = getTaskStatus(_task_id)
    if data is None:
        log.warning("[startTask]当前缓存中不存在此任务:{},放弃处理错误", data)
        return
    data["startTime"] = int(round(time.time() * 1000))
    data["startDate"] = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
    re_pool.hset(aisr_task_status_key, _task_id, json.dumps(data))


def runningTask(_task_id):
    data = getTaskStatus(_task_id)
    if data is None:
        log.warning("[runningTask]当前缓存中不存在此任务:{},放弃处理错误", data)
        return
    data["status"] = task_status.get("running")
    data["runningTime"] = time.time()
    data["runningDate"] = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
    re_pool.hset(aisr_task_status_key, _task_id, json.dumps(data))


def updateTaskStatus(_task_id, status):
    data = getTaskStatus(_task_id)
    if data is None:
        log.warning("[updateTaskStatus]当前缓存中不存在此任务:{},放弃处理错误", data)
        return
    data["status"] = status
    re_pool.hset(aisr_task_status_key, _task_id, json.dumps(data))


def getTaskStatus(_task_id):
    data = re_pool.hget(aisr_task_status_key, _task_id)
    if data is not None:
        return json.loads(data)
    else:
        return data


def restartTodoRunTask(_task_id=None):
    task_list_data = re_pool.hgetall(aisr_task_status_key)
    for tkd in task_list_data:
        key = bytes.decode(tkd)
        tk_val = json.loads(task_list_data[tkd])
        # log.info("当前key:{}结果:{}", key, tk_val)
        if _task_id is not None:
            if key == _task_id:
                h_data = re_pool.hget(aisr_task_data_key, _task_id)
                if h_data is not None and tk_val["status"] == task_status.get("todo"):
                    task_data_str = bytes.decode(h_data, "utf-8")
                    log.info("当前需要把任务:{}重新装回队列...", key)
                    lPush(aisr_queue_key, task_data_str)
        else:
            h_data = re_pool.hget(aisr_task_data_key, key)
            if h_data is not None and tk_val["status"] == task_status.get("todo"):
                task_data_str = bytes.decode(h_data, "utf-8")
                log.info("当前需要把任务:{}重新装回队列...", key)
                lPush(aisr_queue_key, task_data_str)

    return None


def overTaskStatus(_task_id):
    data = getTaskStatus(_task_id)
    if data is None:
        log.warning("[errorTaskStatus]当前缓存中不存在此任务:{},放弃处理错误", data)
        return
    # if data["status"] == task_status.get("error"):
    #     log.warning("[errorTaskStatus]任务执行过程中出现异常:{},暂时无需删除错误信息", data)
    #     return
    re_pool.hdel(aisr_task_status_key, _task_id)
    re_pool.hdel(aisr_task_data_key, _task_id)
    re_pool.delete(aisr_send_mail_key + _task_id)


def buildSendData(_task_id):
    _task_data = re_pool.hget(aisr_task_data_key, _task_id)
    _task_data_str = bytes.decode(_task_data, "utf-8")
    _pro_name = None
    _creator = None
    if _task_data is not None:
        json_data = json.loads(_task_data)
        # 项目名称
        _pro_name = json_data.get("projectName") if json_data.get("projectName") else None
        # 创建人
        _creator = json_data.get("creator") if json_data.get("creator") else None
    return _task_data_str, _pro_name, _creator


def errorTaskStatus(_task_id, _msg, send_mail=True):
    _status_data = getTaskStatus(_task_id)
    if _status_data is None:
        log.warning("[errorTaskStatus]当前缓存中不存在此任务:{},放弃处理错误", _status_data)
        return
    # 暂时打开所有错误都发送邮件功能
    # send_mail = _status_data["status"] == task_status.get("running")
    # 如果单个任务超过10个邮件就不再发了.发多了也没啥意义
    if incr(_task_id) > 10:
        send_mail = False
    _status_data["status"] = task_status.get("error")
    _status_data["errorTime"] = int(round(time.time() * 1000))
    _status_data["errorDate"] = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
    _status_data["errorMsg"] = _msg
    re_pool.hset(aisr_task_status_key, _task_id, json.dumps(_status_data))
    # 发送邮件通知
    if send_mail:
        try:
            log.info("开始发送错误邮件...")
            _task_data_str, _pro_name, _creator = buildSendData(_task_id)
            sendMail(_msg, _task_data_str, _project_name=_pro_name, _creator=_creator)
        except Exception as e:
            log.error("邮件发送错误:{}", str(e))


def incr(taskId):
    return re_pool.incr(aisr_send_mail_key + taskId)


if __name__ == "__main__":
    queue_key = "aisr:park_task_queue"
    value = "{\"taskId\": \"1\"}"
    # data = json.loads(value)
    # for x in range(100):
    #     data["taskId"] = str(int(data["taskId"]) + int(x))
    #     value = json.dumps(data)
    #     # log.info("入队列key:" + queue_key + ",入队列value:" + value + str(x))
    #     lPush(queue_key, value)
    # msg = "TypeError: descriptor 'encode' for 'str' objects doesn't apply to a 'bytes' object"
    # taskId = "1131473191623214108672"
    # # task_data = re_pool.hget(aisr_task_data_key, taskId)
    # # task_data_str = bytes.decode(task_data, "utf-8")
    # errorTaskStatus(taskId, msg)
    # for x in range(1, 10):
    #     log.info("出队列内容:" + rPop(queue_key))
    # restartTodoRunTask("1301474280540101283840")
    # d = getHash("aisr:park_task_data","4271501943917891948544")



import time
from abc import abstractmethod
from aisrfwk.util import log


# from aisrfwk.util import rocketmq_util

class BaseAisrTask:
    son_implements = []

    # def __new__(cls, *args, **kwargs):
    #     if not hasattr(cls, '_instance'):
    #         cls._instance = super(BaseAisrTask, cls).__new__(cls)
    #     return cls._instances
    # def __int__(self,c):
    #     print(c)

    def addSub(self, class_name):
        self.son_implements.append(class_name)

    @abstractmethod
    def run(self, data):
        try:
            log.info("当前为基类业务逻辑方法,如需接入业务处理.请自行创建业务处理方法,同时继承BaseAisrTask")
            # rocketmq_util.sendResultToServer()
            time.sleep(1)
            # rocketmq_util.sendTaskOverToServer()
        except BaseException as e:
            log.info("基类业务处理发生异常:{}", e)
        pass
